C---  MPI vars
      INTEGER name_proc
      CHARACTER(LEN=MPI_MAX_PROCESSOR_NAME) processor_name

C---  some short cuts
C      INTEGER   MPI_INT,MPI_DOUBLE
C      PARAMETER(MPI_INT=MPI_INTEGER,MPI_DOUBLE=MPI_DOUBLE_PRECISION)
